#------------------------------------------------------------------------
#
#  IBM Software
#  Licensed Material - Property of IBM
#  (C) Copyright International Business Machines Corp. 2010, 2017
#  All Rights Reserved.
#
#  U.S. Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#------------------------------------------------------------------------
#  bogusnet.tcl
#------------------------------------------------------------------------

namespace eval bogusnet {
   variable version       "1.1"
   variable sockstr       ""
}

# Returns a "pseudo-random" MAC address.
#
# The idea is to create a MAC address that will be unique within the subnet, to
# avoid ARP conflicts. Creating a MAC address based on the host IP address seems
# like a good heuristic, since this should ensure that a bogus MAC on one host
# will not collide with a bogus MAC on another host in the same subnet. As a
# secondary approach, we generate a random value for the last octet.  This combined
# scheme should work in most cases.  When this scheme is not sufficient, the user
# can supply a MAC on the init call.
proc bogusnet::default_mac {} {
    set match0 [expr int(rand()*255)]
    catch {
        regexp "\\d+.\\d+.\\d+.(\\d+)" [exec host [exec hostname]] fullmatch match0
    }
    set xx [format "%02x" $match0]
    return "d0:d0:d0:da:da:$xx"
}

# bogus net free tap network interface
proc bogusnet::free_tap {} {
    variable sockstr

    if {$sockstr != ""} {
        # Note: Don't need to run as root if deleting an interface owned by the user
        exec $::env(EXEC_DIR)/snif -d $sockstr
        set sockstr ""
    }
}

# bogus net setup
proc bogusnet::init [list { network 172.20.0 } [list mac [bogusnet::default_mac]] { sim mysim } { mcm 0 }] {
    variable sockstr

    if {$sockstr == ""} {
        # check that /dev/net/tun exists and is writable
        if {![file writable /dev/net/tun]} {
            error "Cannot configure bogusnet.  User has insufficient access to tun device."
        }
        set hostip "$network.1"
        set uid [exec /usr/bin/id -u]

        if {[catch {exec $::env(EXEC_DIR)/snif -c -u $uid $hostip} result]} {
            error "Cannot configure bogusnet.  User cannot execute snif. $result"
        }
        set sockstr $result

        trigger set assoc "QUIT" bogusnet::free_tap
    }

    $sim mcm $mcm bogus net init 0 $mac $sockstr 0 0
}

# bogus net setup using sudo
proc bogusnet::sudo_init [list { network 172.20.0 } [list mac [bogusnet::default_mac]] { sim mysim } { mcm 0 }] {
    variable sockstr

    if {$sockstr == ""} {
        # check that /dev/net/tun exists and is writable
        if {![file writable /dev/net/tun]} {
            error "Cannot configure bogusnet.  User has insufficient access to tun device."
        }
        set hostip "$network.1"
        set uid [exec /usr/bin/id -u]

        if {[catch {exec  /usr/bin/sudo $::env(EXEC_DIR)/snif -c -u $uid $hostip} result]} {
            error "Cannot configure bogusnet.  User cannot execute snif. $result"
        }
        set sockstr $result

        trigger set assoc "QUIT" bogusnet::free_tap
    }

    $sim mcm $mcm bogus net init 0 $mac $sockstr 0 0
}


# bogus net cleanup
proc bogusnet::cleanup  { { sim mysim } { mcm 0 } } {
    $sim mcm $mcm bogus net cleanup
    bogusnet::free_tap
}
